unit ApeCommentDisp;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "ApeCommentDisp.pas".
//
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg,
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  XDOM_2_4, UnicodeUtils,
  StrUtils, Registry, ShDocVw, MSHTML, Classes, Controls, SysUtils,
  Forms, Windows, Variants, ActiveX;

type
  TApeCommentDisplay = class(TCustomControl)
  private
    procedure DisplayDirectly(text,
                              fontName: wideString);
    procedure DisplayUsingADummyFile(text,
                                     fontName: wideString);
    function GetIEVersion: string;
  protected
    FDefaultFile: TFilename;
    FWebBrowser: TWebBrowser;
    FOldIE: Boolean;
  public
    constructor Create(aOwner: TComponent); override;
    function DisplayText(text,
                         fontName: wideString): boolean; virtual;
  published
    property Align;
    property DefaultFile: TFilename read FDefaultFile write FDefaultFile;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Ape', [TApeCommentDisplay
                            ]);
end;

{ TApeCommentDisplay }

constructor TApeCommentDisplay.Create(aOwner: TComponent);
var
  V, V_1, V_2: string;
  Path: TFilename;
begin
  inherited;
  FWebBrowser:= TWebBrowser.Create(self);
  FWebBrowser.Align:= alClient;
  TControl(FWebBrowser).Parent:= self;
  V := GetIEVersion;
  V_1 := LeftStr(V, 2);
  V_2 := LeftStr(V, 9);
  if (V_1 = '4.') or (V_1 = '5.') or (V_2 = '6.0.2462.') or (V_2 = '6.0.2479.') then begin
    FOldIE := true;
    // Create temp directory:
    Path:= ExtractFilePath(Application.EXEName) + 'temp';
    if not DirectoryExists(Path)
      then if not CreateDir(Path)
        then raise Exception.Create('Cannot create '+ Path);
  end else FOldIE := false;
end;

procedure TApeCommentDisplay.DisplayDirectly(text,
                                             fontName: wideString);
var
  v: Variant;
  HTMLDocument: IHTMLDocument2;
begin
  HTMLDocument := FWebBrowser.Document as IHTMLDocument2;
  v := VarArrayCreate([0, 0], varVariant);
  v[0]:=  '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">' +
          '<html><head><meta http-equiv="Content-Type" content="text/html; charset=UTF-8"><title></title></head>' +
          '<body style="font-family:''' + fontName + '''" >' + text + '</body></html>';

  HTMLDocument.Write(PSafeArray(TVarData(v).VArray)); // Raises an Access Violation with IE 5.
                                                      // See DisplayUsingADummyFile() for a workaround.
  HTMLDocument.Close;
end;

function TApeCommentDisplay.DisplayText(text,
                                        fontName: wideString): boolean;
begin
  if not assigned(FWebBrowser.Document)       // If there is no document file and ...
    then if DefaultFile <> ''                 // ... if there is a default file specified then ...
      then FWebBrowser.Navigate(DefaultFile); // ... we load the default file first.

  if FOldIE  // = Internet Explorer prior to IE 6.0 RTM is installed
    then DisplayUsingADummyFile(text, fontName)
    else DisplayDirectly(text, fontName);

  result:= true;
end;

procedure TApeCommentDisplay.DisplayUsingADummyFile(text,
                                                    fontName: wideString);
var
  FileStream: TFileStream;
  Path: TFilename;
  S: string;
begin
  Path:= ExtractFilePath(Application.EXEName) + 'temp/~note.html';

  S := '<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">' +
       '<html><head><meta http-equiv="Content-Type" content="text/html; charset=' +
       GetACPEncodingName + 'UTF-8"><title></title></head>' +
       '<body style="font-family:''' + fontName + '''" >' +
       pcdataToStr(text,GetACPEncodingType) + '</body></html>';

  // Write to File:
  FileStream := TFileStream.Create(Path, fmCreate);
  try
    FileStream.WriteBuffer(S[1], Length(S));
  finally
    FileStream.Free;
  end;

  FWebBrowser.Navigate(Path);
end;

function TApeCommentDisplay.GetIEVersion: string;
var
  Registry: TRegistry;
begin
  Registry:= TRegistry.Create(KEY_READ);
  try
    Registry.RootKey := HKEY_LOCAL_MACHINE;
    Registry.OpenKeyReadOnly('\Software\Microsoft\Internet Explorer');
    Result := Registry.ReadString('Version');
  finally
    Registry.Free;
  end;
end;

end.
