unit ApeEditProfileDlg;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "ApeEditProfileDlg.pas".
//
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg,
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  ApePreferences, ApeUtils,
  Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Dialogs;

type
  TEditProfileDialog = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    Panel1: TPanel;
    ProfileNameLabel: TLabel;
    IdentityGroupBox: TGroupBox;
    NameLabeledEdit: TLabeledEdit;
    CacheGroupBox: TGroupBox;
    MemoryCacheLabeledEdit: TLabeledEdit;
    GroupBox1: TGroupBox;
    FontComboBox: TComboBox;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    FProfile: TApeUser;
  public
    function ShowModal(Profile: TApeUser): Integer; reintroduce; overload; virtual;
  end;

var
  EditProfileDialog: TEditProfileDialog;

implementation

{$R *.dfm}

{ TEditProfileDialog }

function TEditProfileDialog.ShowModal(Profile: TApeUser): Integer;
begin
  if not assigned(Profile)
    then raise EInvalidOperation.Create('ActiveUserIndex out of bounds.');

  Screen.ResetFonts;
  FontComboBox.Items:= Screen.Fonts;

  FProfile:= Profile;
  try
    ProfileNameLabel.Caption:= Profile.ProfileName;
    NameLabeledEdit.Text:= Profile.Name;
    MemoryCacheLabeledEdit.Text:= IntToStr(Profile.MemoryCacheSize);
    FontComboBox.Text:= Profile.FontName;
    result:= inherited ShowModal;
  finally
    FProfile:= nil;
  end;
end;

procedure TEditProfileDialog.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  // Note: The constraints fo the values have been tested in FormCloseQuery().

  if ModalResult = mrOK then begin
    FProfile.MemoryCacheSize:= StrToInt(MemoryCacheLabeledEdit.Text);
    FProfile.Name:= NameLabeledEdit.Text;  //  No conversion from local character set is made.  (Postponed to a later APE version)
    FProfile.FontName:= FontComboBox.Text;
  end;
end;

procedure TEditProfileDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  MemCacheSize: integer;
begin
  if ModalResult = mrOK then begin
    try
      MemCacheSize:= StrToInt(MemoryCacheLabeledEdit.Text);
    except
      on EConvertError do  // Raised if MemoryCacheLabeledEdit.Text is not an integer.
        MemCacheSize:= 0;
    end;
    if MemCacheSize < MEMORY_CACHE_SIZE_MIN then begin
      ShowmessageFmt(SMemoryCacheSizeError,[MEMORY_CACHE_SIZE_MIN]);
      MemoryCacheLabeledEdit.Text:= IntToStr(MEMORY_CACHE_SIZE_MIN);
      MemoryCacheLabeledEdit.SetFocus;
      CanClose:= false;
    end;
  end;
end;

end.
