unit ApeHelpDisp;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "ApeHelpDisp.pas".
//
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg,
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  XDOM_2_4, UnicodeUtils, ShDocVw, Contnrs, Classes, Controls, ComCtrls, SysUtils;

type
  TApeHelpFileInfo = class
  private
    FLabelText: wideString;
    FPath: TFilename;
  public
    constructor Create(const path: TFilename;
                       const labelText: wideString); virtual;
    property LabelText: wideString read FLabelText;
    property Path: TFilename read FPath;
  end;

  TApeHelpFileInfoList = class
  private
    FList: TObjectList;
    function GetItem(Index: Integer): TApeHelpFileInfo;
    function GetCount: Integer;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    function Add(path: TFilename;
                 labelText: wideString): Integer; virtual;
    procedure Delete(Index: Integer); virtual;
    procedure Insert(Index: Integer;
                     path: TFilename;
                     labelText: wideString); virtual;
    property Count: Integer read GetCount;
    property Items[Index: Integer]: TApeHelpFileInfo read GetItem; default;
  end;

  TApeHelpDisplay = class(TCustomControl)
  protected
    HelpFileInfoList: TApeHelpFileInfoList;
    FTabControl: TTabControl;
    FWebBrowser: TWebBrowser;
    procedure ApeHelpDisplayTabChange(Sender: TObject); virtual;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    procedure Add(path: TFilename;
                  labelText: wideString); virtual;
    function Display(Index: Integer): boolean; virtual;
    procedure Home; virtual;
  published
    property Align;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Ape', [TApeHelpDisplay
                            ]);
end;

{ TApeHelpFileInfo }

constructor TApeHelpFileInfo.Create(const path: TFilename;
                                    const labelText: wideString);
begin
  FPath:= path;
  FLabelText:= labelText;
end;

{ TApeHelpFileInfoList }

constructor TApeHelpFileInfoList.Create;
begin
  FList:= TObjectList.Create;
end;

destructor TApeHelpFileInfoList.Destroy;
begin
  FList.Destroy;
  inherited;
end;

function TApeHelpFileInfoList.Add(path: TFilename;
                                  labelText: wideString): Integer;
begin
  result:= FList.Add(TApeHelpFileInfo.Create(path,labelText));
end;

procedure TApeHelpFileInfoList.Delete(Index: Integer);
begin
  FList.Delete(Index);
end;

function TApeHelpFileInfoList.GetCount: Integer;
begin
  result:= FList.Count;
end;

function TApeHelpFileInfoList.GetItem(Index: Integer): TApeHelpFileInfo;
begin
  result:= TApeHelpFileInfo(FList.Items[Index]);
end;

procedure TApeHelpFileInfoList.Insert(Index: Integer;
                                      path: TFilename;
                                      labelText: wideString);
begin
  FList.Insert(Index,TApeHelpFileInfo.Create(path,labelText));
end;

{ TApeHelpDisplay }

constructor TApeHelpDisplay.Create(aOwner: TComponent);
begin
  inherited;
  FTabControl:= TTabControl.Create(self);
  FTabControl.Parent:= self;
  FTabControl.Align:= alClient;
  FTabControl.OnChange:= ApeHelpDisplayTabChange;
  FWebBrowser:= TWebBrowser.Create(FTabControl);
  TControl(FWebBrowser).Parent:= FTabControl;
  FWebBrowser.Align:= alClient;
  HelpFileInfoList:= TApeHelpFileInfoList.Create;
end;

destructor TApeHelpDisplay.Destroy;
begin
  HelpFileInfoList.Free;
  inherited;
end;

procedure TApeHelpDisplay.Add(path: TFilename;
                              labelText: wideString);
begin
  HelpFileInfoList.add(path,labelText);
  FTabControl.Tabs.Add(UTF16ToIso8859_1Str(labelText));
end;

function TApeHelpDisplay.Display(Index: Integer): boolean;
begin
  if Index < HelpFileInfoList.count then begin
    FTabControl.TabIndex:= Index;
    result:= true;
    ApeHelpDisplayTabChange(FTabControl);
  end else result := false;
end;

procedure TApeHelpDisplay.ApeHelpDisplayTabChange(Sender: TObject);
var
  index: integer;
begin
  index:= (Sender as TTabControl).TabIndex;
  if index > -1
    then FWebBrowser.Navigate(FilenameToUriStr(HelpFileInfoList[Index].path,[fuPlainColon]));
end;

procedure TApeHelpDisplay.Home;
begin
  ApeHelpDisplayTabChange(FTabControl);
end;

end.
