unit ApeNewProfileDlg;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "ApeNewProfileDlg.pas".
//
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg,
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  ApePreferences,
  Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Mask, Dialogs;

type
  TNewProfileDialog = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    Bevel1: TBevel;
    ProfileNameEdit: TMaskEdit;
    Label1: TLabel;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
  private
    FUserPreferences: TApePreferences;
    function GetProfilePath: TFilename;
  public
    property ProfilePath: TFilename read GetProfilePath;
    property UserPreferences: TApePreferences read FUserPreferences write FUserPreferences;
  end;

var
  NewProfileDialog: TNewProfileDialog;

implementation

{$R *.dfm}

procedure TNewProfileDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if ModalResult = mrOK then begin
    if ProfileNameEdit.Text = '' then begin
      showMessage('Please enter a profile name.');
      CanClose:= false;
    end else begin
      if DirectoryExists(ProfilePath) then begin
        showMessage('This profile already exists. Please enter another profile name.');
        CanClose:= false;
      end;
    end;
  end;
end;

procedure TNewProfileDialog.FormShow(Sender: TObject);
begin
  if not assigned(UserPreferences)
    then raise EInvalidOperation.Create('UserPreferences not specified.');
  if UserPreferences.UsersDirectory = ''
    then raise EInvalidOperation.Create('UserPreferences.Directory not specified.');
end;

function TNewProfileDialog.GetProfilePath: TFilename;
begin
  if not assigned(UserPreferences)
    then raise EInvalidOperation.Create('UserPreferences not specified.');
  if UserPreferences.UsersDirectory = ''
    then raise EInvalidOperation.Create('UserPreferences.Directory not specified.');

  result:= concat(UserPreferences.UsersDirectory, ProfileNameEdit.Text);
end;

end.
