unit InfoEditDlg;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with 
// the License. You may obtain a copy of the License at 
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// The Original Code is "InfoEditDlg.pas".
// 
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg, 
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
// 
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which 
// case the provisions of the GPL are applicable instead of those above. 
// If you wish to allow use of your version of this file only under the 
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  XDOM_2_4, UnicodeUtils, ApeUtils,
  Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, ToolWin, ImgList,
  LanguageAndTextVSTFrame, ApeUrnListFrame;

type
  TInfoEditDialog = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    GroupBox1: TGroupBox;
    AvailabilityStatusLabel: TLabel;
    AvailabilityStatusComboBox: TComboBox;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    Label1: TLabel;
    AboutUrnEdit: TEdit;
    Panel1: TPanel;
    Panel2: TPanel;
    OKBtn: TButton;
    CancelBtn: TButton;
    PageControl2: TPageControl;
    TabSheet5: TTabSheet;
    TabSheet6: TTabSheet;
    TabSheet7: TTabSheet;
    TabSheet8: TTabSheet;
    DomImplementation1: TDomImplementation;
    SourceDescLTEditor: TApeLanguageAndTextEditor;
    AvailabilityLTEditor: TApeLanguageAndTextEditor;
    LabelLTEditor: TApeLanguageAndTextEditor;
    FullTitleLTEditor: TApeLanguageAndTextEditor;
    DisplayPriorityUrnList: TApeUrnListFrame;
    AuthorList: TApeUrnListFrame;
    procedure Panel1Resize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure AvailabilityStatusComboBoxChange(Sender: TObject);
    procedure AboutUrnEditChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FDisplayFontName: TFontName;
    function GetAvailabilityElement: TdomElement;
    function GetDisplayPriorityElement: TdomElement;
    function GetFileDescDocument: TdomDocument;
    function GetFileDescElement: TdomElement;
    function GetNotesStmtElement: TdomElement;
    function GetPublicationStmtElement: TdomElement;
    function GetRenderingStmtElement: TdomElement;
    function GetSourceDescElement: TdomElement;
    function GetTitleStmtElement: TdomElement;
  protected
    function DoStrToWideStrConversion(S: string): wideString; virtual;
    function DoWideStrToStrConversion(S: wideString): string; virtual;
    procedure SetDisplayFontName(const Value: TFontName); virtual;
    procedure UpdateDisplay; virtual;
    procedure UpdateDisplayPriority; virtual;
    procedure UpdateNotesStmt; virtual;
    procedure UpdatePublicationStmt; virtual;
    procedure UpdateSourceDesc; virtual;
    procedure UpdateTitleStmt; virtual;
    property AvailabilityElement: TdomElement read GetAvailabilityElement;
    property DisplayPriorityElement: TdomElement read GetDisplayPriorityElement;
    property FileDescDocument: TdomDocument read GetFileDescDocument;
    property NotesStmtElement: TdomElement read GetNotesStmtElement;
    property PublicationStmtElement: TdomElement read GetPublicationStmtElement;
    property RenderingStmtElement: TdomElement read GetRenderingStmtElement;
    property SourceDescElement: TdomElement read GetSourceDescElement;
    property TitleStmtElement: TdomElement read GetTitleStmtElement;
  public
    procedure InitializeFileDesc(const ape_fileDesc: TdomElement); virtual;
    property DisplayFontName: TFontName read FDisplayFontName write SetDisplayFontName;
    property FileDescElement: TdomElement read GetFileDescElement;
  end;

var
  InfoEditDialog: TInfoEditDialog;

implementation

{$R *.dfm}

procedure TInfoEditDialog.Panel1Resize(Sender: TObject);
begin
  OKBtn.Left:= (Panel1.Width div 2) - OKBtn.Width - 2;
  CancelBtn.Left:= (Panel1.Width div 2) + 2;
end;

procedure TInfoEditDialog.InitializeFileDesc(const ape_fileDesc: TdomElement);
begin
  // Make local copy of subtree
  with FileDescDocument do begin
    clear;
    if assigned(ape_fileDesc)
      then appendChild(importNode(ape_fileDesc,true) as TdomElement);

      // else xxx  not yet implemented;
  end;

  UpdateDisplay;
end;

function TInfoEditDialog.DoStrToWideStrConversion(S: string): wideString;
begin
  result:= strToPcdata(trim(S),GetACPEncodingType);
end;

function TInfoEditDialog.DoWideStrToStrConversion(S: wideString): string;
begin
  result:= trim(pcdataToStr(S,GetACPEncodingType));
end;

procedure TInfoEditDialog.UpdateDisplay;
begin
  UpdateTitleStmt;
  UpdatePublicationStmt;
  UpdateDisplayPriority;
  UpdateSourceDesc;
  UpdateNotesStmt;
end;

procedure TInfoEditDialog.UpdateDisplayPriority;
var
  ape_displayRole: TdomElement;
begin
  with DisplayPriorityUrnList.Lines do begin
    BeginUpdate;
    Clear;
    try
      if assigned(DisplayPriorityElement) then begin
        ape_displayRole:= DisplayPriorityElement.getFirstChildElementNS(xmlns_ape,'displayRole');
        while assigned(ape_displayRole) do begin
          add(ape_displayRole.getAttributeNSNormalizedValue(xmlns_ape,'uid'));
          ape_displayRole:= ape_displayRole.getNextSiblingElementNS(xmlns_ape,'displayRole');
        end;
      end;
    finally
      EndUpdate;
    end;
  end;
end;

procedure TInfoEditDialog.UpdateNotesStmt;
var
  ape_notesStmt,ape_note: TdomElement;
  ape_type: TdomAttr;
  newAboutUrn: wideString;
begin
  newAboutUrn:= '';

  ape_notesStmt:= NotesStmtElement; // Avoid calling GetNotesStmtElement() twice
  if assigned(ape_notesStmt) then begin

    // Traverse <ape:note ...> tags
    ape_note:= ape_notesStmt.getFirstChildElementNS(xmlns_ape,'note');
    while assigned(ape_note) do begin
      ape_type:= ape_note.getAttributeNodeNS(xmlns_ape,'type');
      if assigned(ape_type)
        then if ape_type.value = 'about' then begin
          newAboutUrn:= ape_note.textContent;
          break;
        end;
      ape_note:= ape_note.getNextSiblingElementNS(xmlns_ape,'note');
    end; {while ...}

  end; {if assigned(ape_notesStmt) ...}

  AboutUrnEdit.Text:= DoWideStrToStrConversion(newAboutUrn);
end;

procedure TInfoEditDialog.UpdatePublicationStmt;
var
  ape_availability: TdomElement;
  ape_status: TdomAttr;
  newAvailabilityStatus: wideString;
begin
  ape_availability:= AvailabilityElement; // Avoid calling GetAvailabiltyElement() twice

  // Set availability text:
  AvailabilityLTEditor.InitializeDisplay(ape_availability,
                                         xmlns_ape,'ape:p',
                                         '','','');

  // Set availability status:
  newAvailabilityStatus:= '';
  if assigned(ape_availability) then begin
    ape_status:= ape_availability.getAttributeNodeNS(xmlns_ape,'status');
    if assigned(ape_status)
      then newAvailabilityStatus:= ape_status.value;
  end;
  AvailabilityStatusComboBox.Text:= DoWideStrToStrConversion(newAvailabilityStatus);
end;

procedure TInfoEditDialog.UpdateSourceDesc;
begin
  SourceDescLTEditor.InitializeDisplay(SourceDescElement,
                                       xmlns_ape,'ape:p',
                                       '','','');
end;

procedure TInfoEditDialog.UpdateTitleStmt;
var
  ape_author: TdomElement;
  ape_titleStmt: TdomElement;
begin
  ape_titleStmt:= TitleStmtElement; // Avoid calling GetTitleStmtElement() repeatedly
  FullTitleLTEditor.InitializeDisplay(ape_titleStmt,
                                      xmlns_ape,'ape:title',
                                      xmlns_ape,'ape:type','full');
  LabelLTEditor.InitializeDisplay(ape_titleStmt,
                                  xmlns_ape,'ape:title',
                                  xmlns_ape,'ape:type','label');

  // Update Author List:
  with AuthorList.Lines do begin
    BeginUpdate;
    Clear;
    try
      if assigned(ape_titleStmt) then begin
        ape_author:= ape_titleStmt.getFirstChildElementNS(xmlns_ape,'author');
        while assigned(ape_author) do begin
          add(ape_author.textContent);
          ape_author:= ape_author.getNextSiblingElementNS(xmlns_ape,'author');
        end;
      end;
    finally
      EndUpdate;
    end;
  end;
end;

function TInfoEditDialog.GetAvailabilityElement: TdomElement;
var
  PublicationStmtEl: TdomElement;
begin
  PublicationStmtEl:= PublicationStmtElement; // Avoid calling GetPublicationStmtElement() twice
  if assigned(PublicationStmtEl)
    then result:= PublicationStmtEl.getFirstChildElementNS(xmlns_ape,'availability')
    else result:= nil;
end;

function TInfoEditDialog.GetDisplayPriorityElement: TdomElement;
var
  RenderingStmtEl: TdomElement;
begin
  RenderingStmtEl:= RenderingStmtElement; // Avoid calling GetRenderingStmtElement() twice
  if assigned(RenderingStmtEl)
    then result:= RenderingStmtEl.getFirstChildElementNS(xmlns_ape,'displayPriority')
    else result:= nil;
end;

function TInfoEditDialog.GetFileDescDocument: TdomDocument;
begin
  result:= nil;
  with DomImplementation1.documents do
    if length > 0 then result:= item(0) as TdomDocument;
end;

function TInfoEditDialog.GetFileDescElement: TdomElement;
var
  fileDescDoc: TdomDocument;
begin
  fileDescDoc:= FileDescDocument;  // Avoid calling GetFileDescDocument() twice
  if assigned(fileDescDoc)
    then result:= fileDescDoc.documentElement
    else result:= nil;
end;

function TInfoEditDialog.GetNotesStmtElement: TdomElement;
var
  FileDescEl: TdomElement;
begin
  FileDescEl:= FileDescElement; // Avoid calling GetFileDescElement() twice
  if assigned(FileDescEl)
    then result:= FileDescEl.getFirstChildElementNS(xmlns_ape,'notesStmt')
    else result:= nil;
end;

function TInfoEditDialog.GetPublicationStmtElement: TdomElement;
var
  FileDescEl: TdomElement;
begin
  FileDescEl:= FileDescElement; // Avoid calling GetFileDescElement() twice
  if assigned(FileDescEl)
    then result:= FileDescEl.getFirstChildElementNS(xmlns_ape,'publicationStmt')
    else result:= nil;
end;

function TInfoEditDialog.GetRenderingStmtElement: TdomElement;
var
  FileDescEl: TdomElement;
begin
  FileDescEl:= FileDescElement; // Avoid calling GetFileDescElement() twice
  if assigned(FileDescEl)
    then result:= FileDescEl.getFirstChildElementNS(xmlns_ape,'renderingStmt')
    else result:= nil;
end;

function TInfoEditDialog.GetSourceDescElement: TdomElement;
var
  FileDescEl: TdomElement;
begin
  FileDescEl:= FileDescElement; // Avoid calling GetFileDescElement() twice
  if assigned(FileDescEl)
    then result:= FileDescEl.getFirstChildElementNS(xmlns_ape,'sourceDesc')
    else result:= nil;
end;

function TInfoEditDialog.GetTitleStmtElement: TdomElement;
var
  FileDescEl: TdomElement;
begin
  FileDescEl:= FileDescElement; // Avoid calling GetFileDescElement() twice
  if assigned(FileDescEl)
    then result:= FileDescEl.getFirstChildElementNS(xmlns_ape,'titleStmt')
    else result:= nil;
end;

procedure TInfoEditDialog.FormCreate(Sender: TObject);
begin
  DomImplementation1.createDocumentNS(xmlns_ape,'ape:fileDesc',nil);
end;

procedure TInfoEditDialog.AvailabilityStatusComboBoxChange(
  Sender: TObject);
var
  ape_publicationStmt,ape_availability: TdomElement;
begin
  ape_publicationStmt:= PublicationStmtElement; // Avoid calling GetPublicationStmtElement() twice
  if assigned(ape_publicationStmt) then begin

    // Search for <ape:availability ...>
    with ape_publicationStmt do begin
      ape_availability:= getFirstChildElementNS(xmlns_ape,'availability');
      if not assigned(ape_availability) then begin
        ape_availability:= ownerDocument.createElementNS(xmlns_ape,'ape:availability');
        appendChild(ape_availability);
      end;
    end;

    ape_availability.setAttributeNS(xmlns_ape,'ape:status',DoStrToWideStrConversion((Sender as TComboBox).Text));

  end; {if ...}
end;

procedure TInfoEditDialog.AboutUrnEditChange(Sender: TObject);
var
  ape_notesStmt,ape_note,about_ape_note: TdomElement;
  ape_type: TdomAttr;
begin
  ape_notesStmt:= NotesStmtElement; // Avoid calling GetNotesStmtElement() twice
  if assigned(ape_notesStmt) then begin

    // Search for <ape:note ape:type="about" ...> tag
    about_ape_note:= nil;
    ape_note:= ape_notesStmt.getFirstChildElementNS(xmlns_ape,'note');
    while assigned(ape_note) do begin
      ape_type:= ape_note.getAttributeNodeNS(xmlns_ape,'type');
      if assigned(ape_type)
        then if ape_type.value = 'about' then begin
          about_ape_note:= ape_note;
          break;
        end;
      ape_note:= ape_note.getNextSiblingElementNS(xmlns_ape,'note');
    end; {while ...}

    if not assigned(about_ape_note) then begin
      with ape_notesStmt do begin
        about_ape_note:= ownerDocument.createElementNS(xmlns_ape,'ape:note');
        about_ape_note.setAttributeNS(xmlns_ape,'ape:type','about');
        appendChild(about_ape_note);
      end;
    end;

    with about_ape_note do begin
      clear;
      appendChild(ownerDocument.createTextNode(DoStrToWideStrConversion((Sender as TEdit).Text)));
    end;

  end; {if assigned(ape_notesStmt) ...}
end;

procedure TInfoEditDialog.FormClose(Sender: TObject;
  var Action: TCloseAction);
var
  ape_author: TdomElement;
  ape_displayRole: TdomElement;
  ape_titleStmt: TdomElement;
  i: integer;
begin
  // Confirm Display Priority:
  if assigned(DisplayPriorityElement) then begin
    DisplayPriorityElement.clear;
    with DisplayPriorityUrnList.Lines do begin
      for i:= 0 to pred(count) do begin
        ape_displayRole:= DisplayPriorityElement.ownerDocument.createElementNS(xmlns_ape,'ape:displayRole');
        ape_displayRole.setAttributeNS(xmlns_ape,'ape:uid',wideStrings[i]);
        DisplayPriorityElement.appendChild(ape_displayRole);
      end;
    end;
  end;

  // Confirm Author Names:
  ape_titleStmt:= TitleStmtElement; // Avoid calling GetTitleStmtElement() repeatedly
  if assigned(ape_titleStmt) then begin
    // Delete old <ape:author> entries:
    ape_author:= ape_titleStmt.getFirstChildElementNS(xmlns_ape,'author');
    while assigned(ape_author) do begin
      ape_titleStmt.removeChild(ape_author);
      ape_author.ownerDocument.freeAllNodes(TdomNode(ape_author));
      ape_author:= ape_titleStmt.getFirstChildElementNS(xmlns_ape,'author');
    end;
    // Append new <ape:author> entries:
    with AuthorList.Lines do begin
      for i:= 0 to pred(count) do begin
        ape_author:= ape_titleStmt.ownerDocument.createElementNS(xmlns_ape,'ape:author');
        ape_author.appendChild(ape_titleStmt.ownerDocument.createTextNode(wideStrings[i]));
        ape_titleStmt.appendChild(ape_author);
      end;
    end;
  end;
end;

procedure TInfoEditDialog.SetDisplayFontName(const Value: TFontName);
begin
  if FDisplayFontName <> Value then begin
    FDisplayFontName:= Value;
    FullTitleLTEditor.Font.Name:= Value;
    LabelLTEditor.Font.Name:= Value;
    AuthorList.Font.Name:= Value;
    AvailabilityStatusComboBox.Font.Name:= Value;
    AvailabilityLTEditor.Font.Name:= Value;
    SourceDescLTEditor.Font.Name:= Value;
    AboutUrnEdit.Font.Name:= Value;
    DisplayPriorityUrnList.Font.Name:= Value;
  end;
end;

end.
