unit StructureDisplayFrame;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with 
// the License. You may obtain a copy of the License at 
// "http://www.mozilla.org/MPL/"
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// The Original Code is "StructureDisplayFrame.pas".
// 
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg, 
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  XDOM_2_4, UnicodeUtils, FileDescDisplayFrame, ApeUtils, ApePreferences,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  VirtualTrees, ExtCtrls, OleCtrls, SHDocVw, ComCtrls, Buttons, StdCtrls,
  ActnList, StdActns, ToolWin, ImgList, ExtActns,
  ApeVirtualStructures, ApeCommentDisp, IdBaseComponent, IdComponent,
  IdTCPConnection, IdTCPClient, IdHTTP, IdAntiFreezeBase, IdAntiFreeze;

type
  TStructureDisplayError = (
    sdInvalidURISchemeError,
    sdInvalidURLError,
    sdLoadingCanceledError,
    sdNetworkError,
    sdProcessingError,
    sdTimeOutError
  );

  TApeStructureDisplayErrorEvent = procedure(Sender: TObject; error: TStructureDisplayError) of object;

  TStructureDisplayState = (
    sdSaveAsCanceled // indicating that the last StructureSaveAs execution was canceled
  );

  TStructureDisplayStates = set of TStructureDisplayState;

  TStructDispFrame = class(TFrame)
    DomImpl: TDomImplementation;
    XmlToDomParser: TXmlToDomParser;
    XmlStandardDomReader: TXmlStandardDomReader;
    XmlDocBuilder: TXmlDocBuilder;
    ActionList1: TActionList;
    StructureOpen: TFileOpen;
    NavigationToolBar: TToolBar;
    UrlEdit: TEdit;
    OpenToolButton: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    StopToolButton: TToolButton;
    ToolButton6: TToolButton;
    ShowStructureToolButton: TToolButton;
    ShowInfoToolButton: TToolButton;
    EnabledButtonsImageList: TImageList;
    DisabledButtonsImageList: TImageList;
    StructureClose: TAction;
    NewDocToolButton: TToolButton;
    InfoEdit: TAction;
    FileToolBar: TToolBar;
    StructureNew: TAction;
    SaveAllStructuresToolButton: TToolButton;
    SaveStructureToolButton: TToolButton;
    ToolButton2: TToolButton;
    StructureSave: TAction;
    StructureSaveAll: TAction;
    StructureSaveAs: TFileSaveAs;
    ToolButton5: TToolButton;
    SaveAsToolButton: TToolButton;
    CloseToolButton: TToolButton;
    EditToolButton: TToolButton;
    ToggleEdit: TAction;
    ItemEdit: TAction;
    EditToolBar: TToolBar;
    EditInfoToolButton: TToolButton;
    ItemNoteToolButton: TToolButton;
    StructureDisplay: TAction;
    InfoDisplay: TAction;
    ToolButton1: TToolButton;
    UpToolButton: TToolButton;
    DownToolButton: TToolButton;
    PullOutToolButton: TToolButton;
    IndentToolButton: TToolButton;
    UpAction: TAction;
    DownAction: TAction;
    AddAction: TAction;
    DeleteAction: TAction;
    PullOutAction: TAction;
    IndentAction: TAction;
    AddToolButton1: TToolButton;
    RemoveToolButton: TToolButton;
    ToolButton7: TToolButton;
    StructureCloseAll: TAction;
    ApeStructurePageControl1: TApeStructurePageControl;
    DomToXmlParser: TDomToXmlParser;
    IdHTTP1: TIdHTTP;
    procedure UrlEditKeyPress(Sender: TObject; var Key: Char);
    procedure StopLoading(Sender: TObject);
    procedure StructureOpenAccept(Sender: TObject);
    procedure NavigationToolBarResize(Sender: TObject);
    procedure StructureCloseExecute(Sender: TObject);
    procedure InfoEditExecute(Sender: TObject);
    procedure ItemEditExecute(Sender: TObject);
    procedure StructureSaveUpdate(Sender: TObject);
    procedure StructureSaveAllUpdate(Sender: TObject);
    procedure ActionList1Update(Action: TBasicAction;
      var Handled: Boolean);
    procedure StructureSaveAsAccept(Sender: TObject);
    procedure StructureSaveExecute(Sender: TObject);
    procedure StructureSaveAsBeforeExecute(Sender: TObject);
    procedure StructureOpenBeforeExecute(Sender: TObject);
    procedure StructureSaveAllExecute(Sender: TObject);
    procedure ToggleEditExecute(Sender: TObject);
    procedure StructureDisplayExecute(Sender: TObject);
    procedure InfoDisplayExecute(Sender: TObject);
    procedure StructureCloseUpdate(Sender: TObject);
    procedure ToggleEditUpdate(Sender: TObject);
    procedure StructureDisplayUpdate(Sender: TObject);
    procedure InfoDisplayUpdate(Sender: TObject);
    procedure ItemEditUpdate(Sender: TObject);
    procedure UpActionUpdate(Sender: TObject);
    procedure DownActionUpdate(Sender: TObject);
    procedure AddActionUpdate(Sender: TObject);
    procedure DeleteActionUpdate(Sender: TObject);
    procedure UpActionExecute(Sender: TObject);
    procedure DownActionExecute(Sender: TObject);
    procedure DeleteActionExecute(Sender: TObject);
    procedure AddActionExecute(Sender: TObject);
    procedure IndentActionUpdate(Sender: TObject);
    procedure PullOutActionUpdate(Sender: TObject);
    procedure PullOutActionExecute(Sender: TObject);
    procedure IndentActionExecute(Sender: TObject);
    procedure StructureSaveAsCancel(Sender: TObject);
    procedure StructureCloseAllUpdate(Sender: TObject);
    procedure StructureCloseAllExecute(Sender: TObject);
    procedure StructureNewExecute(Sender: TObject);
    procedure ApeStructurePageControl1FocusChanged(Sender: TObject;
      Index: Integer);
    procedure ApeStructurePageControl1InfoModified(Sender: TObject;
      Index: Integer);
    procedure ApeStructurePageControl1ItemModfied(Sender: TObject;
      Index: Integer);
    procedure ApeStructurePageControl1Change(Sender: TObject);
  private
    FActiveResourceMap: TdomWideStringList;
    FDocumentToBeSaved: integer;
    FOutputFormat: TdomEncodingType;
    FOnError: TApeStructureDisplayErrorEvent; // Occurs after a processing error.
    FOnFocusChanged: TNotifyEvent;            // Occurs after the focus was moved to another item.
    FOnInfoModified: TNotifyEvent;            // Occurs after the struture info has been modified.
    FOnActivePageChange: TNotifyEvent;        // Occurs after another tab has been activated (or the last active tab has been removed).
    FTemplateFile: TFilename;
    function GetDocumentToBeSaved: integer;
  protected
    FStates: TStructureDisplayStates;
    FUserPreferences: TApePreferences;
    procedure DoActivePageChange; virtual;
    procedure DoError(const error: TStructureDisplayError); virtual;
    procedure DoFocusChanged(const index: integer); virtual;
    procedure DoInfoModified; virtual;
    procedure DoResourceMapChanged; virtual;
    function GetActiveCommentText: wideString; virtual;
    function GetActiveDisplayPriority: TApeDisplayPriorityList; virtual;
    function GetActiveFocusElement: TdomElement; virtual;
    function GetActivePageIndex: integer; virtual;
    function GetDisplayFontName: TFontName; virtual;
    function GetLanguage: TIso639LanguageCode; virtual;
    function HasValidFileURL(const doc: TdomDocument): boolean; virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure SaveStructureToFile(const Path: TFilename;
                                  const StructureIndex: integer;
                                  const Encoding: TdomEncodingType); virtual;
    procedure SetDisplayFontName(const Value: TFontName); virtual;
    procedure SetLanguage(const Value: TIso639LanguageCode); virtual;
    procedure SetUserPreferences(const Value: TApePreferences); virtual;
    procedure SetOutputFormat(const Value: TdomEncodingType); virtual;
    property ActiveFocusElement: TdomElement read GetActiveFocusElement;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function AddStructure(const structureDoc: TdomDocument): boolean; virtual;
    function CloseActiveStructure: boolean; virtual;
    function CloseAllStructures: boolean; virtual;
    function LoadStructureFromFile(const Path: TFilename;
                                   const SysId: wideString): boolean; virtual;
    function LoadStructureFromString(const S: string;
                                     const SysId: wideString): boolean; virtual;
    function OpenDirectory(const directory: TFilename): boolean; virtual;
    function OpenUrl(const url: string): boolean; virtual;
    property ActiveCommentText: wideString read GetActiveCommentText;
    property ActiveDisplayPriority: TApeDisplayPriorityList read GetActiveDisplayPriority;
    property ActivePageIndex: integer read GetActivePageIndex;
    property ActiveResourceMap: TdomWideStringList read FActiveResourceMap;
    property States: TStructureDisplayStates read FStates;
  published
    property DisplayFontName: TFontName read GetDisplayFontName write SetDisplayFontName;
    property Language: TIso639LanguageCode read GetLanguage write SetLanguage default iso639_en;
    property OutputFormat: TdomEncodingType read FOutputFormat write SetOutputFormat default etUTF_8;
    property TemplateFile: TFilename read FTemplateFile write FTemplateFile;
    property UserPreferences: TApePreferences read FUserPreferences write SetUserPreferences;
    property OnActivePageChange: TNotifyEvent read FOnActivePageChange write FOnActivePageChange;
    property OnError: TApeStructureDisplayErrorEvent read FOnError write FOnError;
    property OnFocusChanged: TNotifyEvent read FOnFocusChanged write FOnFocusChanged;
    property OnInfoModified: TNotifyEvent read FOnInfoModified write FOnInfoModified;
  end;

const
  TIME_OUT: integer = 30000; // Timeout for HTTP client

type
  TApeStructureDisplay = class(TStructDispFrame);

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Ape', [TApeStructureDisplay]);
end;

{$R *.DFM}

type

TTreeData = record
  element: TdomElement;
end;

PTreeData = ^TTreeData;

constructor TStructDispFrame.Create(AOwner: TComponent);
begin
  inherited;
  FOutputFormat:= etUTF_8;
  FDocumentToBeSaved:= -1;
  FStates:= [];
  Language:= iso639_en;
  FActiveResourceMap:= TdomWideStringList.create;
end;

destructor TStructDispFrame.Destroy;
begin
  FActiveResourceMap.free;
  inherited;
end;

function TStructDispFrame.AddStructure(const structureDoc: TdomDocument): boolean;
var
  structureDocCopy: TdomDocument;
begin
  if not assigned(structureDoc)
    then begin result:= false; exit; end;

  // Parse into namespace aware document tree:
  structureDocCopy:= DomImpl.createDocument('structureDisplay',nil);
  try
    structureDocCopy.clear;
    XmlDocBuilder.referenceNode:= structureDocCopy;
    result:= XmlStandardDomReader.parse(structureDoc);
    if result then begin
      structureDocCopy.systemId:= structureDoc.systemId;
      result:= ApeStructurePageControl1.AddStructure(structureDocCopy);
    end;
  finally
    DomImpl.freeDocument(structureDocCopy);
  end;
end;

function TStructDispFrame.CloseActiveStructure: boolean;
var
  modificationWarning: string;
begin
  result:= false;
  with ApeStructurePageControl1 do begin
    if ActivePageIndex = -1 then exit;

    if Modified[ActivePageIndex] then begin
      // Document was modified --> Save it to disk?
      if SystemId[ActivePageIndex] = ''
        then modificationWarning:= WideFormat(SSaveChanges,[SSaveWhatDefault])
        else modificationWarning:= WideFormat(SSaveChanges,[SystemId[ActivePageIndex]]);
      case MessageDlg(modificationWarning,mtConfirmation,[mbYes,mbNo,mbCancel],0) of
        mrYes: begin
          try
            FDocumentToBeSaved:= ActivePageIndex;
            StructureSave.Execute;  // Call the StructureSave Action.
          finally
            FDocumentToBeSaved:= -1;
          end;
          if sdSaveAsCanceled in FStates then exit;
        end;
        mrCancel: exit;
      end;
    end; {if ...}

    result:= CloseStructure(ApeStructurePageControl1.ActivePageIndex);

    if PageCount = 0
      then EditToolbar.Visible:= false;

  end; {with ...}
end;

function TStructDispFrame.CloseAllStructures: boolean;
var
  noToAllFlag: boolean;
  modificationWarning: string;
begin
  noToAllFlag:= false;
  with ApeStructurePageControl1 do begin
    while PageCount > 0 do begin
      if (not noToAllFlag) and Modified[pred(PageCount)] then begin
        // Document was modified --> Save it to disk?
      if SystemId[pred(PageCount)] = ''
          then modificationWarning:= WideFormat(SSaveChanges,[SSaveWhatDefault])
          else modificationWarning:= WideFormat(SSaveChanges,[SystemId[pred(PageCount)]]);
        case MessageDlg(modificationWarning,mtConfirmation,[mbYes,mbNo,mbCancel,mbNoToAll],0) of
          mrYes: begin
            try
              FDocumentToBeSaved:= pred(PageCount);
              StructureSave.Execute;  // Call the StructureSave Action.
            finally
              FDocumentToBeSaved:= -1;
            end;
            if sdSaveAsCanceled in FStates then begin
              // If canceled then set active Tab to display the current document:
              ActivePageIndex:= pred(PageCount);
              break;
            end;
          end;
          mrCancel: begin
            // If canceled then set active Tab to display the current document:
            ActivePageIndex:= pred(PageCount);
            break;
          end;
          mrNoToAll:
            noToAllFlag:= true;
        end;
      end;
      CloseStructure(pred(PageCount));
    end; {while ...}
    result:= PageCount = 0
  end; {with ...}
end;

procedure TStructDispFrame.DoActivePageChange;
begin
  DoResourceMapChanged;
  if assigned(FOnActivePageChange)
    then FOnActivePageChange(self);
end;

procedure TStructDispFrame.DoError(const error: TStructureDisplayError);
begin
  if assigned(FOnError)
    then FOnError(self,error);
end;

procedure TStructDispFrame.DoFocusChanged(const index: integer);
begin
  DoResourceMapChanged;
  if assigned(FOnFocusChanged)
    then if index = ActivePageIndex
      then FOnFocusChanged(self);
end;

procedure TStructDispFrame.DoInfoModified;
begin
  if assigned(FOnInfoModified)
    then FOnInfoModified(self);
end;

procedure TStructDispFrame.DoResourceMapChanged;
var
  ape_resource,ape_resourceMap: TdomElement;
  xlink_href: TdomAttr;
begin
  FActiveResourceMap.clear;

  if assigned(ActiveFocusElement) then begin

    // Search for <ape:resourceMap ...>
    ape_resourceMap:= ActiveFocusElement.getFirstChildElementNS(xmlns_ape,'resourceMap');

    // Search for all <ape:ape_resource ...>
    if assigned(ape_resourceMap) then begin
      ape_resource:= ape_resourceMap.getFirstChildElementNS(xmlns_ape,'resource');
      while assigned(ape_resource) do begin
        xlink_href:= ape_resource.getAttributeNodeNS(xmlns_xlink,'href');
        if assigned(xlink_href)
          then FActiveResourceMap.Add(xlink_href.value);
        ape_resource:= ape_resource.getNextSiblingElementNS(xmlns_ape,'resource');
      end; {while ...}
    end; {if ...}

  end; {if ...}
end;

function TStructDispFrame.GetActiveCommentText: wideString;
// Returns a sequence of comment paragraphs wrapped in <p> tags.
var
  ape_comment: TdomElement;
  i,j: integer;
  n: TdomNode;
  S: wideString;
begin
  Result:= '';
  if assigned(ActiveFocusElement) then begin
    ape_comment:= ActiveFocusElement.getFirstChildElementNS(xmlns_ape,'comment');
    if assigned(ape_comment) then begin
      for i:= 0 to pred(ape_comment.childNodes.length) do begin
        n:= ape_comment.childNodes.item(i);
        if n.nodeType = ntElement_Node then begin
          if n.nodeName = 'ape:p' then begin
            Result:= concat(Result,'<p>');
            for j:= 0 to pred(n.childNodes.length) do begin
              DomToXmlParser.writeToWideString(n.childNodes.item(j),S);
              Result:= concat(Result,S);
            end;
            Result:= concat(Result,'</p>');
          end;
        end;
      end;
    end;
  end;
end;

function TStructDispFrame.GetActiveDisplayPriority: TApeDisplayPriorityList;
begin
  with ApeStructurePageControl1 do
    if ActivePageIndex = -1
      then result:= nil
      else result:= DisplayPriority[ActivePageIndex];
end;

function TStructDispFrame.GetActiveFocusElement: TdomElement;
begin
  with ApeStructurePageControl1 do
    if ActivePageIndex = -1
      then result:= nil
      else result:= FocusElement[ActivePageIndex];
end;

function TStructDispFrame.GetActivePageIndex: integer;
begin
  result:= ApeStructurePageControl1.ActivePageIndex;
end;

function TStructDispFrame.GetDisplayFontName: TFontName;
begin
  result:= ApeStructurePageControl1.DisplayFontName;
end;

function TStructDispFrame.GetLanguage: TIso639LanguageCode;
begin
  result:= ApeStructurePageControl1.Language;
end;

function TStructDispFrame.HasValidFileURL(const doc: TdomDocument): boolean;
// Returns 'true', if the specified Document has a
// systemId with a valid file-scheme URL.
// Returns 'false' otherwise or when doc is 'nil'.
var
  uriAnalyzer: TUriWideStrAnalyzer;
begin
  result:= false;
  if not assigned(doc) then exit;
  with doc do begin
    if isUriAbsoluteURIWideStr(systemId) then begin
      uriAnalyzer:= TUriWideStrAnalyzer.create;
      try
        with uriAnalyzer do begin
          setUriReference(systemId);
          if UriScheme = 'file' then result:= true;
        end;
      finally
        uriAnalyzer.free;
      end; {try ... finally ...}
    end; {if ...}
  end; {with ...}
end;

function TStructDispFrame.OpenDirectory(const directory: TFilename): boolean;
var
  Error: integer;
  F: TSearchRec;
begin
  Update;
  result:= false;
  Error:= FindFirst(directory+'*.xml',faAnyFile,F);
  try
    while Error = 0 do begin
      result:= OpenUrl(filenameToUriStr(directory+F.Name,[])) or result; // OpenUrl must be put first, because of short-circuit evaluation.
      Error:= FindNext(F);
    end;
  finally
    FindClose(F);
  end;
  if result then DoActivePageChange;
end;

function TStructDispFrame.OpenUrl(const url: string): boolean;
var
  path: TFilename;
  authority,query,fragment,documentStr: string;
  uriAnalyzer: TUriStrAnalyzer;
begin
  result:= False;
  uriAnalyzer:= TUriStrAnalyzer.create;
  try
    uriAnalyzer.setUriReference(url);
    with uriAnalyzer do begin
      if HasUriScheme then begin
        if UriScheme = 'file' then begin
          if UriStrToFilename(url,path,authority,query,fragment) then begin
            if LoadStructureFromFile(path,url)
              then result:= true
              else doError(sdProcessingError);
          end else doError(sdInvalidURLError);
        end else if UriScheme = 'http' then begin
          StopToolButton.Enabled:= True;
          try
            try
              documentStr:= IdHTTP1.Get(url);
            except
              doError(sdProcessingError);
            end;
            if LoadStructureFromString(documentStr,url)
              then result:= true
              else doError(sdProcessingError);
          finally
            StopToolButton.Enabled:= False;
          end; {try ...}
        end else begin
          doError(sdInvalidURISchemeError);
        end;
      end else begin
        doError(sdInvalidURLError);
      end; {if HasUriScheme ... else ...}
    end; {with ...}
  finally
    uriAnalyzer.free;
  end;
end;

function TStructDispFrame.LoadStructureFromFile(const Path: TFilename;
                                                const SysId: wideString): boolean;
var
  dummyDoc: TdomDocument;
  Save_Cursor:TCursor;
begin
  result:= false;
  if not fileExists(Path)
    then exit;

  Save_Cursor := Screen.Cursor;
  Screen.Cursor := crHourGlass; // Show hourglass cursor
  try
    try
      with XmlToDomParser do begin
        dummyDoc:= FileToDom(Path);
        try
          result:= dummyDoc.prepareASModels;
          if result
            then result:= dummyDoc.validate(erReplace);
          if result then begin
            dummyDoc.systemId:= sysId;
            result:= AddStructure(dummyDoc);
          end;
        finally
          DomImpl.freeDocument(dummyDoc);
          DomImpl.freeUnusedASModels;
        end;
      end;
    except
      result:= false;
    end;
  finally
    Screen.Cursor := Save_Cursor; // Restore cursor
  end;
end;

function TStructDispFrame.LoadStructureFromString(const S: string;
                                                  const SysId: wideString): boolean;
var
  dummyDoc: TdomDocument;
  Save_Cursor:TCursor;
begin
  result:= false;
  if S = '' then exit;

  Save_Cursor := Screen.Cursor;
  Screen.Cursor := crHourGlass; // Show hourglass cursor
  try
    try
      with XmlToDomParser do begin
        dummyDoc:= stringToDom(S);
        try
          result:= dummyDoc.prepareASModels;
          if result
            then result:= dummyDoc.validate(erReplace);
          if result then begin
            dummyDoc.systemId:= sysId;
            result:= AddStructure(dummyDoc);
          end;
        finally
          DomImpl.freeDocument(dummyDoc);
          DomImpl.freeUnusedASModels;
        end;
      end;
    except
      result:= false;
    end;
  finally
    Screen.Cursor := Save_Cursor; // Restore cursor
  end;
end;

procedure TStructDispFrame.Notification(AComponent: TComponent;
                                         Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then begin
    if AComponent = FUserPreferences then begin
      FUserPreferences:= nil;
    end;
  end;
end;

procedure TStructDispFrame.SaveStructureToFile(const Path: TFilename;
                                               const StructureIndex: integer;
                                               const Encoding: TdomEncodingType);
var
  fFileOut: TFileStream;
  outputStr: string;
  outputWideStr: wideString;
begin
  fFileOut:= TFileStream.Create(Path,fmCreate);
  case Encoding of
    etUTF_8: begin
      outputStr := ApeStructurePageControl1.DocumentAsString(StructureIndex);
      fFileOut.Write(outputStr[1],Length(outputStr));
      fFileOut.Free;
    end;
    etUTF_16BE: begin
      outputWideStr := ApeStructurePageControl1.DocumentAsWideString(StructureIndex);
      fFileOut.Write(outputWideStr[1],Length(outputWideStr)*2);
      fFileOut.Free;
    end;
  end;
end;

procedure TStructDispFrame.SetDisplayFontName(const Value: TFontName);
begin
  ApeStructurePageControl1.DisplayFontName:= Value;
end;

procedure TStructDispFrame.SetLanguage(const Value: TIso639LanguageCode);
begin
  ApeStructurePageControl1.Language:= Value;
end;

procedure TStructDispFrame.SetOutputFormat(const Value: TdomEncodingType);
begin
  if not (value in [etUTF_8,etUTF_16BE])
    then raise ENot_Supported_Err.create('Not supported error.');
  FOutputFormat := Value;
end;

procedure TStructDispFrame.SetUserPreferences(const Value: TApePreferences);
begin
  FUserPreferences:= Value;
end;

procedure TStructDispFrame.UrlEditKeyPress(Sender: TObject; var Key: Char);
begin
  if key = #13 then
    if OpenUrl(UrlEdit.Text)
      then DoActivePageChange;
end;

procedure TStructDispFrame.StopLoading(Sender: TObject);
begin
  IdHTTP1.DisconnectSocket;
  StopToolButton.Enabled:= false;
end;

procedure TStructDispFrame.StructureNewExecute(Sender: TObject);
begin
  OpenUrl(filenameToUriStr(TemplateFile,[]));
  with ApeStructurePageControl1 do begin
    SystemId[ActivePageIndex]:= '';
    if assigned(FUserPreferences) then begin
      if assigned(FUserPreferences.ActiveUser)
        then AuthorName[ActivePageIndex]:= FUserPreferences.ActiveUser.Name
        else AuthorName[ActivePageIndex]:= '';
    end else AuthorName[ActivePageIndex]:= '';
  end;
  UrlEdit.Text:= '';
  DoActivePageChange;
end;

procedure TStructDispFrame.NavigationToolBarResize(Sender: TObject);
begin
  UrlEdit.width:= (Sender as TControl).width-139;
end;

procedure TStructDispFrame.StructureCloseUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= ApeStructurePageControl1.PageCount > 0;
end;

procedure TStructDispFrame.StructureCloseAllUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= ApeStructurePageControl1.PageCount > 1;
end;

procedure TStructDispFrame.ToggleEditUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= ApeStructurePageControl1.PageCount > 0;
  (Sender as TAction).Checked:= EditToolBar.Visible;
end;

procedure TStructDispFrame.ToggleEditExecute(Sender: TObject);
begin
  EditToolbar.Visible:= (Sender as TAction).Checked;
  if ToggleEdit.Checked and not StructureDisplay.Checked
    then StructureDisplay.Execute;
end;

procedure TStructDispFrame.StructureDisplayExecute(Sender: TObject);
begin
  ApeStructurePageControl1.ShowInfo:= false;
end;

procedure TStructDispFrame.InfoDisplayExecute(Sender: TObject);
begin
  if ToggleEdit.Checked
    then ToggleEdit.Execute;
  ApeStructurePageControl1.ShowInfo:= true;
end;

procedure TStructDispFrame.InfoDisplayUpdate(Sender: TObject);
begin
  (Sender as TAction).Checked:= ApeStructurePageControl1.ShowInfo;
end;

procedure TStructDispFrame.ItemEditUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= ApeStructurePageControl1.CanEdit;
end;

procedure TStructDispFrame.StructureDisplayUpdate(Sender: TObject);
begin
  (Sender as TAction).Checked:= not ApeStructurePageControl1.ShowInfo;
end;

procedure TStructDispFrame.StructureCloseExecute(Sender: TObject);
begin
  if CloseActiveStructure
    then DoActivePageChange;
end;

procedure TStructDispFrame.StructureCloseAllExecute(Sender: TObject);
var
  oldActivePageIndex: integer;
begin
  oldActivePageIndex:= ApeStructurePageControl1.ActivePageIndex;
  CloseAllStructures;
  if oldActivePageIndex <> ApeStructurePageControl1.ActivePageIndex
    then DoActivePageChange;
end;

procedure TStructDispFrame.StructureSaveAsCancel(Sender: TObject);
begin
  // Activate the sdCancelSaveAs state, to indicate that closing a document should be aborted
  include(FStates,sdSaveAsCanceled);
end;

function TStructDispFrame.GetDocumentToBeSaved: integer;
begin
  if FDocumentToBeSaved > -1
    then result:= FDocumentToBeSaved
    else result:= ApeStructurePageControl1.ActivePageIndex;
end;

procedure TStructDispFrame.InfoEditExecute(Sender: TObject);
begin
  ApeStructurePageControl1.InfoEdit;
end;

procedure TStructDispFrame.ItemEditExecute(Sender: TObject);
begin
  ApeStructurePageControl1.ItemEdit;
end;

procedure TStructDispFrame.StructureSaveUpdate(Sender: TObject);
var
  index: integer;
begin
  index:= GetDocumentToBeSaved; // Avoid calling GetDocumentToBeSaved() twice
  if index > -1
    then (Sender as TAction).Enabled:= ApeStructurePageControl1.modified[index]
    else (Sender as TAction).Enabled:= false;
end;

procedure TStructDispFrame.StructureSaveAllUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= ApeStructurePageControl1.modifiedCount > 0;
end;

procedure TStructDispFrame.ActionList1Update(Action: TBasicAction;
  var Handled: Boolean);
begin
  // StructureSaveAs publishes no OnUpdate event, so we will have to do the update here.
  if action = StructureSaveAs
    then StructureSaveAs.Enabled:= ApeStructurePageControl1.PageCount > 0;
end;

procedure TStructDispFrame.StructureSaveAsAccept(Sender: TObject);
var
  index: integer;
begin
  index:= GetDocumentToBeSaved;
  if index > -1 then begin
    SaveStructureToFile(StructureSaveAs.Dialog.FileName,index,OutputFormat);
    ApeStructurePageControl1.SystemId[index]:= filenameToUriStr(StructureSaveAs.Dialog.FileName,[]);
    ApeStructurePageControl1.modified[index]:= false;
    UrlEdit.Text:= ApeStructurePageControl1.SystemId[index];
  end;
  exclude(FStates,sdSaveAsCanceled);
end;

procedure TStructDispFrame.StructureSaveAsBeforeExecute(Sender: TObject);
begin
  (Sender as TFileSaveAs).Dialog.FileName:= '';
end;

procedure TStructDispFrame.StructureOpenAccept(Sender: TObject);
var
  i: integer;
  ok: boolean;
begin
  Update;
  ok:= false;
  with StructureOpen.Dialog.Files do
    for i:= pred(count) downto 0 do
      ok:= OpenUrl(filenameToUriStr(Strings[i],[])) or ok;  // OpenUrl must be put first, because of short-circuit evaluation.
  if ok then DoActivePageChange;
end;

procedure TStructDispFrame.StructureOpenBeforeExecute(Sender: TObject);
begin
  (Sender as TFileOpen).Dialog.FileName:= '';
end;

procedure TStructDispFrame.StructureSaveAllExecute(Sender: TObject);
var
  i: integer;
begin
  with ApeStructurePageControl1 do begin
    for i:= 0 to pred(PageCount) do begin
      if Modified[i] then begin
        try
          FDocumentToBeSaved:= i;
          StructureSave.Execute;  // Call the StructureSave Action.
        finally
          FDocumentToBeSaved:= -1;
        end;
      end; {if ...}
    end; {for ...}
  end; {with ...}
end;

procedure TStructDispFrame.StructureSaveExecute(Sender: TObject);
var
  path: TFilename;
  authority, query, fragment: string;
  index: integer;
begin
  index:= GetDocumentToBeSaved;
  if index = -1 then exit;
  if uriWideStrToFilename(ApeStructurePageControl1.SystemId[index],path,authority,query,fragment) then begin
    SaveStructureToFile(path,index,OutputFormat);
    ApeStructurePageControl1.modified[index]:= false;
  end else StructureSaveAs.ExecuteTarget(nil);  // Call the StructureSaveAs Action.  The target parameter is of no meaning here, so it is set to 'nil'.
end;

procedure TStructDispFrame.UpActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= ApeStructurePageControl1.CanUp;
end;

procedure TStructDispFrame.DownActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= ApeStructurePageControl1.CanDown;
end;

procedure TStructDispFrame.PullOutActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= ApeStructurePageControl1.CanPullOut;
end;

procedure TStructDispFrame.IndentActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= ApeStructurePageControl1.CanIndent;
end;

procedure TStructDispFrame.AddActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= ApeStructurePageControl1.CanAdd;
end;

procedure TStructDispFrame.DeleteActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:= ApeStructurePageControl1.CanDelete;
end;

procedure TStructDispFrame.UpActionExecute(Sender: TObject);
begin
  ApeStructurePageControl1.Up;
end;

procedure TStructDispFrame.DownActionExecute(Sender: TObject);
begin
  ApeStructurePageControl1.Down;
end;

procedure TStructDispFrame.PullOutActionExecute(Sender: TObject);
begin
  ApeStructurePageControl1.PullOut;
end;

procedure TStructDispFrame.IndentActionExecute(Sender: TObject);
begin
  ApeStructurePageControl1.Indent;
end;

procedure TStructDispFrame.DeleteActionExecute(Sender: TObject);
begin
  ApeStructurePageControl1.Delete;
end;

procedure TStructDispFrame.AddActionExecute(Sender: TObject);
begin
  ApeStructurePageControl1.Add;
end;

procedure TStructDispFrame.ApeStructurePageControl1Change(Sender: TObject);
begin
  DoActivePageChange;
end;

procedure TStructDispFrame.ApeStructurePageControl1FocusChanged(
  Sender: TObject; Index: Integer);
begin
  DoFocusChanged(Index);
end;

procedure TStructDispFrame.ApeStructurePageControl1InfoModified(
  Sender: TObject; Index: Integer);
begin
  DoInfoModified;
end;

procedure TStructDispFrame.ApeStructurePageControl1ItemModfied(
  Sender: TObject; Index: Integer);
begin
  DoFocusChanged(Index);
end;

end.
