unit VTCtrls;

// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with 
// the License. You may obtain a copy of the License at 
// "http://www.mozilla.org/MPL/"
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// The Original Code is "VTCtrls.pas".
// 
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg, 
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2002
// Dieter Khler. All Rights Reserved.
// 
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above. 
// If you wish to allow use of your version of this file only under the 
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

interface

uses
  Classes, StdCtrls, Messages, Controls, Windows, Graphics, Forms,
  XDOM_2_4, UnicodeUtils, VirtualTrees;

type
  TUnicodeStringEditLink = class;
  TVirtualWideStringList = class;

  TVTUnicodeEdit = class(TCustomEdit)
  // This is a modified version of the TVTEdit class from Mike Lischke's
  // VirtualTrees v. 3.3.3.
  private
    FLink: TUnicodeStringEditLink;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure WMChar(var Message: TWMChar); message WM_CHAR;
    procedure WMDestroy(var Message: TWMDestroy); message WM_DESTROY;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure WMKeyDown(var Message: TWMKeyDown); message WM_KEYDOWN;
  protected
    procedure AutoAdjustSize;
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(Link: TUnicodeStringEditLink); reintroduce;
  end;

  TUnicodeStringEditLink = class(TInterfacedObject, IVTEditLink)
  // This is a modified version of the TStringEditLink class from Mike
  // Lischke's VirtualTrees v. 3.3.3.
  private
    FEdit: TVTUnicodeEdit;         // A custom edit control for Unicode.
    FTree: TVirtualWideStringList; // A back reference to the tree calling.
    FNode: PVirtualNode;           // The node to be edited.
    FColumn: TColumnIndex;         // The column of the node.
    FAlignment: TAlignment;
    FTextBounds: TRect;            // Smallest rectangle around the text.
    FStopping: Boolean;            // Set to True when the edit link requests
                                   // stopping the edit action.
  public
    constructor Create;
    destructor Destroy; override;
    function BeginEdit: Boolean; stdcall;
    function CancelEdit: Boolean; stdcall;
    function EndEdit: Boolean; stdcall;
    function GetBounds: TRect; stdcall;
    function PrepareEdit(Tree: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex): Boolean; stdcall;
    procedure ProcessMessage(var Message: TMessage); stdcall;
    procedure SetBounds(R: TRect); stdcall;
  end;

  TVirtualWideStringList = class(TCustomVirtualStringTree)
  private
    FLines : TdomWideStringList;
    procedure GetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
    TextType: TVSTTextType; var CellText: WideString);
    procedure LinesChanged(Sender: TObject);
  protected
    function DoCreateEditor(Node: PVirtualNode; Column: TColumnIndex): IVTEditLink; override;
    function  GetOptionsClass: TTreeOptionsClass; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Canvas;
    property Lines : TdomWideStringList read FLines;
  published
    property Align;
    property Alignment;
    property Anchors;
    property AnimationDuration;
    property AutoExpandDelay;
    property AutoScrollDelay;
    property AutoScrollInterval;
    property Background;
    property BackgroundOffsetX;
    property BackgroundOffsetY;
    property BiDiMode;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelKind;
    property BevelWidth;
    property BorderStyle;
    property ButtonFillMode;
    property ButtonStyle;
    property BorderWidth;
    property ChangeDelay;
    property CheckImageKind;
    property ClipboardFormats;
    property Color;
    property Colors;
    property Constraints;
    property Ctl3D;
    property CustomCheckImages;
    property DefaultNodeHeight;
    property DefaultPasteMode;
    property DefaultText;
    property DragHeight;
    property DragKind;
    property DragImageKind;
    property DragMode;
    property DragOperations;
    property DragSelection;
    property DragType;
    property DragWidth;
    property DrawSelectionMode;
    property EditDelay;
    property Enabled;
    property Font;
    property Header;
    property HintAnimation;
    property HintMode;
    property HotCursor;
    property Images;
    property IncrementalSearch;
    property IncrementalSearchDirection;
    property IncrementalSearchStart;
    property IncrementalSearchTimeout;
    property Indent;
    property LastDropMode;
    property LineMode;
    property LineStyle;
    property Margin;
    property NodeAlignment;
    property NodeDataSize;
    property ParentBiDiMode;
    property ParentColor default False;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property RootNodeCount;
    property ScrollBarOptions;
    property SelectionCurveRadius;
    property ShowHint;
    property StateImages;
    property TabOrder;
    property TabStop default True;
    property TextMargin;
    property TreeOptions;
    property Visible;
    property WantTabs;

    property OnAfterCellPaint;
    property OnAfterItemErase;
    property OnAfterItemPaint;
    property OnAfterPaint;
    property OnBeforeCellPaint;
    property OnBeforeItemErase;
    property OnBeforeItemPaint;
    property OnBeforePaint;
    property OnChange;
    property OnChecked;
    property OnChecking;
    property OnClick;
    property OnCollapsed;
    property OnCollapsing;
    property OnColumnClick;
    property OnColumnDblClick;
    property OnColumnResize;
    property OnCompareNodes;
    property OnCreateDataObject;
    property OnCreateDragManager;
    property OnCreateEditor;
    property OnDblClick;
    property OnDragAllowed;
    property OnDragOver;
    property OnDragDrop;
    property OnEditCancelled;
    property OnEdited;
    property OnEditing;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnExpanded;
    property OnExpanding;
    property OnFocusChanged;
    property OnFocusChanging;
    property OnFreeNode;
    //  property OnGetText;  Not used.  Only internally relevant.
    property OnGetHelpContext;
    property OnGetImageIndex;
    property OnGetHint;
    property OnGetLineStyle;
    property OnGetNodeDataSize;
    property OnGetPopupMenu;
    property OnGetUserClipboardFormats;
    property OnHeaderClick;
    property OnHeaderDblClick;
    property OnHeaderDragged;
    property OnHeaderDraggedOut;
    property OnHeaderDragging;
    property OnHeaderDraw;
    property OnHeaderMouseDown;
    property OnHeaderMouseMove;
    property OnHeaderMouseUp;
    property OnHotChange;
    property OnIncrementalSearch;
    property OnInitChildren;
    property OnInitNode;
    property OnKeyAction;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnLoadNode;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnNewText;
    property OnNodeCopied;
    property OnNodeCopying;
    property OnNodeMoved;
    property OnNodeMoving;
    property OnPaintBackground;
    property OnPaintText;
    property OnRenderOLEData;
    property OnResetNode;
    property OnResize;
    property OnSaveNode;
    property OnScroll;
    property OnShortenString;
    property OnStartDock;
    property OnStartDrag;
    property OnStructureChange;
    property OnUpdating;
  end;

resourcestring
  // Localizable strings.
  SEditLinkIsNil = 'Edit link must not be nil.';

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('APE', [TVirtualWideStringList]);
end;

{ TVTUnicodeEdit }

constructor TVTUnicodeEdit.Create(Link: TUnicodeStringEditLink);
begin
  inherited Create(nil);
  ShowHint := False;
  ParentShowHint := False;
  FLink := Link;
end;

procedure TVTUnicodeEdit.AutoAdjustSize;

// Changes the size of the edit to accomodate as much as possible of its text
// within its container window.
// NewChar describes the next character which will be added to the edit's text.

var
  DC: HDC;
  Size: TSize;
  LastFont: THandle;

begin
  // avoid flicker
  SendMessage(Handle, WM_SETREDRAW, 0, 0);

  DC := GetDC(Handle);
  LastFont := SelectObject(DC, Font.Handle);
  try
    // Read needed space for the current text.
    GetTextExtentPoint32(DC, PChar(Text), Length(Text), Size);
    Inc(Size.cx, 2 * FLink.FTree.TextMargin);

    // Repaint associated node if the edit becomes smaller.
    if Size.cx < Width then
      FLink.FTree.InvalidateNode(FLink.FNode);

    if FLink.FAlignment = taRightJustify then
      FLink.SetBounds(Rect(Left + Width - Size.cx, Top, Left + Width, Top + Height))
    else
      FLink.SetBounds(Rect(Left, Top, Left + Size.cx, Top + Height));
  finally
    SelectObject(DC, LastFont);
    ReleaseDC(Handle, DC);
    SendMessage(Handle, WM_SETREDRAW, 1, 0);
  end;
end;

procedure TVTUnicodeEdit.CNCommand(var Message: TWMCommand);
begin
  if Assigned(FLink) and Assigned(FLink.FTree) and (Message.NotifyCode = EN_UPDATE) and
    not (toGridExtensions in (FLink.FTree.TreeOptions as TStringTreeOptions).MiscOptions) then
    AutoAdjustSize;
end;

procedure TVTUnicodeEdit.CreateParams(var Params: TCreateParams);
begin
  inherited;

  // Only with multiline style we can use the text formatting rectangle.
  // This does not harm formatting as single line control, as we don't use word
  // wrapping.
  Params.Style := Params.Style or ES_MULTILINE;
  with Params do
  begin
    if tsUseThemes in FLink.FTree.TreeStates then
    begin
      Style := Style and not WS_BORDER;
      ExStyle := ExStyle or WS_EX_CLIENTEDGE;
    end
    else
    begin
      Style := Style or WS_BORDER;
      ExStyle := ExStyle and not WS_EX_CLIENTEDGE;
    end;
  end;
end;

procedure TVTUnicodeEdit.WMChar(var Message: TWMChar);
begin
  if not (Message.CharCode in [VK_ESCAPE, VK_TAB]) then
    inherited;
end;

procedure TVTUnicodeEdit.WMDestroy(var Message: TWMDestroy);
begin
  // If editing stopped by other means than accept or cancel then we have to do
  // default processing for pending changes.
  if not FLink.FStopping then
    with FLink, FTree do
    begin
      if (toAutoAcceptEditChange in (TreeOptions as TStringTreeOptions).StringOptions) and Modified then
        DoNewText(FNode, FColumn, FEdit.Caption);
    end;

  inherited;
end;

procedure TVTUnicodeEdit.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  inherited;

  Message.Result := Message.Result or DLGC_WANTTAB;
end;

procedure TVTUnicodeEdit.WMKeyDown(var Message: TWMKeyDown);

// Handles some control keys. 

begin
  case Message.CharCode of
    // pretend these keycodes were send to the tree
    VK_ESCAPE:
      FLink.FTree.WndProc(TMessage(Message));
    VK_RETURN:
      FLink.FTree.DoEndEdit;
    VK_UP:
      begin
        Message.CharCode := VK_LEFT;
        inherited;
      end;
    VK_DOWN:
      begin
        Message.CharCode := VK_RIGHT;
        inherited;
      end;
  else
    inherited;
  end;
end;

{ TVirtualWideStringList }

constructor TVirtualWideStringList.Create(AOwner: TComponent);
begin
  inherited;
  with TStringTreeOptions(TreeOptions) do
   begin
     PaintOptions := PaintOptions - [toShowRoot, toShowTreeLines, toShowButtons]
                                  + [toHideFocusRect];
     SelectionOptions := SelectionOptions + [toFullRowSelect];
   end;
  TextMargin := 0;
  OnGetText := GetText;
  FLines := TdomWideStringList.Create;
  Lines.OnChange := LinesChanged;
end;

destructor TVirtualWideStringList.Destroy;
begin
  FLines.Free;
  inherited;
end;

function TVirtualWideStringList.DoCreateEditor(Node: PVirtualNode;
  Column: TColumnIndex): IVTEditLink;
begin
  if Assigned(OnCreateEditor) then
  begin
    Result := nil;
    OnCreateEditor(Self, Node, Column, Result);
    if Result = nil then
      raise EVirtualTreeError.CreateHelp(SEditLinkIsNil, hcTFEditLinkIsNil);
  end
  else
  begin
    // Enable generic label editing support if the application does not have
    // own editors.
    Result := TUnicodeStringEditLink.Create;
  end;
end;

function TVirtualWideStringList.GetOptionsClass: TTreeOptionsClass;
begin
  Result := TStringTreeOptions;
end;

procedure TVirtualWideStringList.GetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
begin
  if Node.Index < Cardinal(FLines.Count)
    then CellText := FLines[Node.Index]
    else CellText := 'Overflow Error';
end;

procedure TVirtualWideStringList.LinesChanged(Sender: TObject);
begin
  RootNodeCount := FLines.Count;
end;

{ TUnicodeStringEditLink }

constructor TUnicodeStringEditLink.Create;
begin
  inherited;
  FEdit := TVTUnicodeEdit.Create(Self);
  with FEdit do
  begin
    Visible := False;
    BorderStyle := bsSingle;
    AutoSize := False;
  end;
end;

destructor TUnicodeStringEditLink.Destroy;
begin
  FEdit.Free;
  inherited;
end;

function TUnicodeStringEditLink.BeginEdit: Boolean;
// Notifies the edit link that editing can start now. Descentants may cancel
// node edit by returning False.
begin
  Result := not FStopping;
  if Result then
  begin
    FEdit.AutoAdjustSize;
    FEdit.Show;
    FEdit.SelectAll;
    FEdit.SetFocus;
  end;
end;

function TUnicodeStringEditLink.CancelEdit: Boolean;
begin
  Result := not FStopping;
  if Result then
  begin
    FStopping := True;
    FEdit.Hide;
    FTree.CancelEditNode;
  end;
end;

function TUnicodeStringEditLink.EndEdit: Boolean;
begin
  Result := not FStopping;
  if Result then
  try
    FStopping := True;
    if FEdit.Modified then
      FTree.DoNewText(FNode, FColumn, strToPcdata(FEdit.Caption,GetACPEncodingType)); 
    FEdit.Hide;
  except
    FStopping := False;
    raise;
  end;
end;

function TUnicodeStringEditLink.GetBounds: TRect;
begin
  Result := FEdit.BoundsRect;
end;

function TUnicodeStringEditLink.PrepareEdit(Tree: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex): Boolean;
// Transforms characters unknown to the editor's font into character
// references and retrieves the true text bounds from the owner tree.
var
  EscapedText: String;
begin
  Result := Tree is TVirtualWideStringList;
  if Result then
  begin
    FTree := Tree as TVirtualWideStringList;
    FNode := Node;
    FColumn := Column;
    // initial size, font and text of the node:
    EscapedText:= pcdataToStr(FTree.Text[Node, Column],GetACPEncodingType);
    FEdit.Font.Assign(FTree.Font);
    FEdit.Font.Color := clBlack;
    FEdit.Caption := EscapedText;
    FEdit.Parent := Tree;
    FTextBounds := FTree.GetDisplayRect(Node, Column, True, True);

    if Column <= NoColumn then
    begin
      FEdit.BidiMode := FTree.BidiMode;
      FAlignment := FTree.Alignment;
    end
    else
    begin
      FEdit.BidiMode := FTree.Header.Columns[Column].BidiMode;
      FAlignment := FTree.Header.Columns[Column].Alignment;
    end;

    if FEdit.BidiMode <> bdLeftToRight then
      ChangeBidiModeAlignment(FAlignment);
  end;
end;

procedure TUnicodeStringEditLink.ProcessMessage(var Message: TMessage);
begin
  FEdit.WindowProc(Message);
end;

procedure TUnicodeStringEditLink.SetBounds(R: TRect);
// Sets the outer bounds of the edit control and the actual edit area in the
// control.
var
  Offset: Integer;
begin
  if not FStopping then
  begin
    with R do
    begin
      // Set the edit's bounds but make sure there's a minimum width and the
      // right border does not extend beyond the parent's left/right border.
      if Left < 0 then
        Left := 0;
      if Right - Left < 30 then
      begin
        if FAlignment = taRightJustify then
          Left := Right - 30
        else
          Right := Left + 30;
      end;
      if Right > FTree.ClientWidth then
        Right := FTree.ClientWidth;
      FEdit.BoundsRect := R;

      // The selected text shall exclude the text margins and be centered
      // vertically.  We have to take out the two pixel border of the edit
      // control as well as a one pixel "edit border" the control leaves
      // around the (selected) text.
      R := FEdit.ClientRect;
      Offset := 2;
      if tsUseThemes in FTree.TreeStates then
        Inc(Offset);
      InflateRect(R, -FTree.TextMargin + Offset, Offset);
      OffsetRect(R, 0, FTextBounds.Top - FEdit.Top);

      SendMessage(FEdit.Handle, EM_SETRECTNP, 0, Integer(@R));
    end;
  end;
end;

end.
